// painterView.cpp : implementation of the CPainterView class
//

#include "stdafx.h"
#include "painter.h"

#include "painterDoc.h"
#include "painterView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPainterView

IMPLEMENT_DYNCREATE(CPainterView, CView)

BEGIN_MESSAGE_MAP(CPainterView, CView)
	//{{AFX_MSG_MAP(CPainterView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_TOOLS_DRAWFREEHAND, OnToolsDrawfreehand)
	ON_COMMAND(ID_TOOLS_ELLIPSE, OnToolsEllipse)
	ON_COMMAND(ID_TOOLS_FILLFIGURE, OnToolsFillfigure)
	ON_COMMAND(ID_TOOLS_LINE, OnToolsLine)
	ON_COMMAND(ID_TOOLS_RECTANGLE, OnToolsRectangle)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_DRAWFREEHAND, OnUpdateToolsDrawfreehand)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_ELLIPSE, OnUpdateToolsEllipse)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_FILLFIGURE, OnUpdateToolsFillfigure)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_LINE, OnUpdateToolsLine)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_RECTANGLE, OnUpdateToolsRectangle)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPainterView construction/destruction

CPainterView::CPainterView()
{
	// TODO: add construction code here
	MakeAllFlagsFalse();
}

CPainterView::~CPainterView()
{
}

BOOL CPainterView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.lpszClass = AfxRegisterWndClass(CS_DBLCLKS, AfxGetApp()->LoadStandardCursor(IDC_CROSS), (HBRUSH)(COLOR_WINDOW+1), AfxGetApp()->LoadIcon(IDR_MAINFRAME));
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CPainterView drawing

void CPainterView::OnDraw(CDC* pDC)
{
	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	HMETAFILE MetaFileHandle = pDoc->pMetaFileDC->Close();
	pDC->PlayMetaFile(MetaFileHandle);
	CMetaFileDC* ReplacementMetaFile = new CMetaFileDC();
	ReplacementMetaFile->Create();
	ReplacementMetaFile->PlayMetaFile(MetaFileHandle);
	DeleteMetaFile(MetaFileHandle);
	delete pDoc->pMetaFileDC;
	pDoc->pMetaFileDC = ReplacementMetaFile;

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CPainterView printing

BOOL CPainterView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CPainterView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CPainterView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CPainterView diagnostics

#ifdef _DEBUG
void CPainterView::AssertValid() const
{
	CView::AssertValid();
}

void CPainterView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CPainterDoc* CPainterView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CPainterDoc)));
	return (CPainterDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPainterView message handlers

void CPainterView::OnLButtonDown(UINT nFlags, CPoint point) 
{
    Anchor.x = point.x;
    Anchor.y = point.y;
    OldPoint.x = Anchor.x;
    OldPoint.y = Anchor.y;

	CView::OnLButtonDown(nFlags, point);
}

void CPainterView::OnLButtonUp(UINT nFlags, CPoint point) 
{

	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	DrawTo.x = point.x;
	DrawTo.y = point.y;

	CClientDC* pDC = new CClientDC(this);

	if(bLineFlag){
		pDC->MoveTo(Anchor.x, Anchor.y);
		pDC->LineTo(DrawTo.x, DrawTo.y);
		pDoc->pMetaFileDC->MoveTo(Anchor.x, Anchor.y);
		pDoc->pMetaFileDC->LineTo(DrawTo.x, DrawTo.y);
	}

	if(bRectangleFlag){
		pDC->SelectStockObject(NULL_BRUSH);
		pDC->Rectangle(Anchor.x, Anchor.y, DrawTo.x, DrawTo.y);
		pDoc->pMetaFileDC->SelectStockObject(NULL_BRUSH);
		pDoc->pMetaFileDC->Rectangle(Anchor.x, Anchor.y, DrawTo.x, DrawTo.y);
	}

	if(bEllipseFlag){
		pDC->SelectStockObject(NULL_BRUSH);
		pDC->Ellipse(Anchor.x, Anchor.y, DrawTo.x, DrawTo.y);
		pDoc->pMetaFileDC->SelectStockObject(NULL_BRUSH);
		pDoc->pMetaFileDC->Ellipse(Anchor.x, Anchor.y, DrawTo.x, DrawTo.y);
	}

	if(bFillFlag){
		pDC->SelectStockObject(BLACK_BRUSH);
		pDC->FloodFill(Anchor.x, Anchor.y, RGB(0, 0, 0));
		pDoc->pMetaFileDC->SelectStockObject(BLACK_BRUSH);
		pDoc->pMetaFileDC->FloodFill(Anchor.x, Anchor.y, RGB(0, 0, 0));
	}

	delete pDC;

	CView::OnLButtonUp(nFlags, point);
}

void CPainterView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int nOldMode;

	CClientDC* pDC = new CClientDC(this);

	if((nFlags && MK_LBUTTON) && bDrawFlag){
		pDC->MoveTo(Anchor.x, Anchor.y);
		pDC->LineTo(point.x, point.y);
		pDoc->pMetaFileDC->MoveTo(Anchor.x, Anchor.y);
		pDoc->pMetaFileDC->LineTo(point.x, point.y);
		Anchor.x = point.x;
		Anchor.y = point.y;
	}

	if((nFlags && MK_LBUTTON) && bLineFlag){
		nOldMode = pDC->GetROP2();
		pDC->SetROP2(R2_NOT);
		pDC->MoveTo(Anchor.x, Anchor.y);
		pDC->LineTo(OldPoint.x, OldPoint.y);
		pDC->MoveTo(Anchor.x, Anchor.y);
		pDC->LineTo(point.x, point.y);
		OldPoint.x = point.x;
		OldPoint.y = point.y;
		pDC->SetROP2(nOldMode);
    }

    if((nFlags && MK_LBUTTON) && bRectangleFlag){
		CClientDC dc(this);
		nOldMode = pDC->GetROP2();
		pDC->SetROP2(R2_NOT);
		pDC->SelectStockObject(NULL_BRUSH);
		pDC->Rectangle(OldPoint.x, OldPoint.y, Anchor.x, Anchor.y);
		pDC->Rectangle(Anchor.x, Anchor.y, point.x, point.y);
		OldPoint.x = point.x;
		OldPoint.y = point.y;
		pDC->SetROP2(nOldMode);
	}

    if((nFlags && MK_LBUTTON) && bEllipseFlag){
		CClientDC dc(this);
		nOldMode = pDC->GetROP2();
		pDC->SetROP2(R2_NOT);
		pDC->SelectStockObject(NULL_BRUSH);
		pDC->Ellipse(OldPoint.x, OldPoint.y, Anchor.x, Anchor.y);
		pDC->Ellipse(Anchor.x, Anchor.y, point.x, point.y);
		OldPoint.x = point.x;
		OldPoint.y = point.y;
		pDC->SetROP2(nOldMode);
	}
	
	delete pDC;

	CView::OnMouseMove(nFlags, point);
}

void CPainterView::MakeAllFlagsFalse()
{
 	bDrawFlag = false;
	bLineFlag = false;
	bRectangleFlag = false;
	bEllipseFlag = false;
	bFillFlag = false;
}


void CPainterView::OnToolsDrawfreehand() 
{
	MakeAllFlagsFalse();
	bDrawFlag = true;	
	
}

void CPainterView::OnToolsEllipse() 
{
	MakeAllFlagsFalse();
	bEllipseFlag = true;	
	
}

void CPainterView::OnToolsFillfigure() 
{
	MakeAllFlagsFalse();
	bFillFlag = true;	
	
}

void CPainterView::OnToolsLine() 
{
	MakeAllFlagsFalse();
	bLineFlag = true;	
	
}

void CPainterView::OnToolsRectangle() 
{
	MakeAllFlagsFalse();
	bRectangleFlag = true;	
	
}

void CPainterView::OnUpdateToolsDrawfreehand(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(bDrawFlag);
}

void CPainterView::OnUpdateToolsEllipse(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(bEllipseFlag);
	
}

void CPainterView::OnUpdateToolsFillfigure(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(bFillFlag);
	
}

void CPainterView::OnUpdateToolsLine(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(bLineFlag);
	
}

void CPainterView::OnUpdateToolsRectangle(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(bRectangleFlag);
	
}

void CPainterView::OnFileSave() 
{
	// TODO: Add your command handler code here
	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	HMETAFILE MetaFileHandle = pDoc->pMetaFileDC->Close();
	CopyMetaFile(MetaFileHandle, "painter.wmf");

	CMetaFileDC* ReplacementMetaFile = new CMetaFileDC();
	ReplacementMetaFile->Create();
	ReplacementMetaFile->PlayMetaFile(MetaFileHandle);
	DeleteMetaFile(MetaFileHandle);
	delete pDoc->pMetaFileDC;
	pDoc->pMetaFileDC = ReplacementMetaFile;

}

void CPainterView::OnFileOpen() 
{
	// TODO: Add your command handler code here
	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	HMETAFILE MetaFileHandle = GetMetaFile("painter.wmf");

	CMetaFileDC* ReplacementMetaFile = new CMetaFileDC();
	ReplacementMetaFile->Create();
	ReplacementMetaFile->PlayMetaFile(MetaFileHandle);
	DeleteMetaFile(MetaFileHandle);
	delete pDoc->pMetaFileDC;
	pDoc->pMetaFileDC = ReplacementMetaFile;
	
	Invalidate();
}

void CPainterView::OnFileNew() 
{
	// TODO: Add your command handler code here
	CPainterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CMetaFileDC* ReplacementMetaFile = new CMetaFileDC();
	ReplacementMetaFile->Create();
	delete pDoc->pMetaFileDC;
	pDoc->pMetaFileDC = ReplacementMetaFile;
	
	Invalidate();

}
